package fiber

import (
	"errors"
	"html/template"
	"strconv"
	"strings"
	"time"

	"github.com/gofiber/fiber/v2"
	"github.com/gofiber/template/html/v2"

	"git.dmitriygnatenko.ru/dima/dmitriygnatenko-v2/internal/mapper"
	"git.dmitriygnatenko.ru/dima/dmitriygnatenko-v2/internal/services/i18n"
)

func Now() time.Time {
	return time.Now()
}

func NoEscape(str string) template.HTML {
	return template.HTML(str)
}

func Concat(parts ...string) string {
	return strings.Join(parts, "")
}

func GridSep(i, l int) bool {
	i++
	return i%3 == 0 && i != l
}

func Version(sp ServiceProvider) func() string {
	return func() string {
		return strconv.FormatUint(uint64(sp.ConfigService().StaticVersion()), 10)
	}
}

func Trans(lang i18n.Language, key string) string {
	return i18n.T(lang, key)
}

func Link(lang i18n.Language, link string) string {
	if lang == i18n.Default {
		return link
	}

	return "/" + string(lang) + link
}

func getViewsEngine(sp ServiceProvider) *html.Engine {
	engine := html.New(templatesPath, ".html")

	engine.AddFunc("now", Now)
	engine.AddFunc("noescape", NoEscape)
	engine.AddFunc("concat", Concat)
	engine.AddFunc("gridsep", GridSep)
	engine.AddFunc("version", Version(sp))
	engine.AddFunc("trans", Trans)
	engine.AddFunc("link", Link)

	return engine
}

func getErrorHandler() fiber.ErrorHandler {
	return func(fctx *fiber.Ctx, err error) error {
		lang := mapper.LanguageFromContext(fctx)
		errCode := fiber.StatusInternalServerError
		var e *fiber.Error
		if errors.As(err, &e) {
			errCode = e.Code
		}

		var renderData fiber.Map

		if errCode == fiber.StatusNotFound {
			renderData = fiber.Map{
				"pageTitle": i18n.T(lang, "page_not_found_err_title"),
				"code":      fiber.StatusNotFound,
				"text":      i18n.T(lang, "page_not_found_err_desc"),
			}
		} else {
			renderData = fiber.Map{
				"pageTitle": i18n.T(lang, "internal_err_title"),
				"code":      fiber.StatusInternalServerError,
				"text":      i18n.T(lang, "internal_err_desc"),
			}
		}

		renderData["headTitle"] = i18n.T(lang, "head_title")

		return fctx.Render("error", renderData, "_layout")
	}
}
